/*************************************************************************
 *
 *  ts_list.c - Library of routines for the user to write for
 *                readout and buffering of events using
 *                a Linux VME controller.
 *
 */

/* Event Buffer definitions */
#define MAX_EVENT_POOL     400
#define MAX_EVENT_LENGTH   1024*10      /* Size in Bytes */

/* Define Interrupt source and address */
#define TS_SOURCE
#define TS_MODE TS_POLL
#define TS_ADDR 0xed0000

#include "linuxvme_list.c"


/* function prototype */
void rocTrigger(int arg);

void
rocDownload()
{
  /* Initialize modules here */

  printf("rocDownload: User Download Executed\n");

}

void
rocPrestart()
{
  /* Program Trigger Supervisor and Configure VME Modules Here */
  tsReset(0);

  tsCsr2Set(TS_CSR2_LOCK_ALL);  /* ROC Lock mode on all BRANCHES */
  tsEnableInput(0xfff,0);       /* Enable all Trigger inputs in non-strobe mode */
  tsRoc(0,0,0,0);               /* Enable ACK 0,1 on BRANCH 1  and ACK 2 on Branch 2*/

  /* Synchronization programming */
  tsSync(100);                      /* schedule Sync every 100th physics trigger */
  tsCsr2Set(TS_CSR2_ENABLE_SYNC);   /* Enable Scheduled syncs */ 

  /* Set nominal Level 2/3 timer values */
  tsTimerWrite(TS_TIMER_L2A,0x05);   /* Level 2 Timer 40ns/count */
  tsTimerWrite(TS_TIMER_L3A,0x05);   /* Level 3 Timer 40ns/count */

  /* Front End Busy timer 40ns/count */
  tsTimerWrite(TS_TIMER_FB,250);         /* 250 = 10 microsec */   
  tsCsr2Set(TS_CSR2_ENABLE_FB_TIMER);    /* Enable FEB Timer */

  /*  Construct TS memory data  ---  in the following model, all trigger patterns 
      that form the memory address are assigned to trigger class 1.  For those 
      trigger patterns with a single hit, the ROC code is set to be the trigger 
      input number.  Otherwise, the ROC code is set to 0xE.  All LEVEL 1 ACCEPT 
      signals are asserted for every pattern.  */
   
  tsMemInit();

  /* Fix special cases - both inputs 1 and 2 firing - type 13 (0xd) 
     all L1 accept outputs firing 0xff03 */
  tsMemWrite(3,0xdff03);

  /* Set specific input prescale factors */
  tsPrescale(1,0);
  tsPrescale(2,0);

  printf("rocPrestart: User Prestart Executed\n");
}

void
rocGo()
{
  /* Enable modules, if needed, here */

  /* Interrupts/Polling enabled after return of rocGo() */
}

void
rocEnd()
{
  /* Disable modules here, if needed */

  printf("rocEnd: Ended after %d events\n",tsGetIntCount());
}

void
rocTrigger(int arg)
{
  int ii;
  unsigned int event_ty=0, event_no=0;

  event_ty = EVTYPE;
  event_no = EVNUM;

  /* Open an event bank of banks with event type EVTYPE (obtained from TS) */
  EVENTOPEN(EVTYPE, BT_BANK);

  /* Open Bank number 5 (for example), Bank of 32bit unsigned integers */
  BANKOPEN(5,BT_UI4,0);
  *dma_dabufp++ = LSWAP(tsGetIntCount()); /* Insert the trigger count */

  /* Insert some data here */
  for(ii=0; ii<20; ii++) 
    {
      *dma_dabufp++ = LSWAP(ii);
    }

  *dma_dabufp++ = LSWAP(0xd00dd00d); /* Insert another data word */
  BANKCLOSE;

  EVENTCLOSE;

}

void
rocCleanup()
{
  /* Put any clean up code that should be done before the next readout list is downloaded */

}
